@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -18,7 +18,7 @@
  */
 
 /*
- For windows debuging #define UNICODE
+ For windows debugging #define UNICODE
 */
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -114,6 +114,9 @@ static WCHAR   _currDirW [SS_MAX_DIR_LENGTH];
 static char    _currDir [SS_MAX_DIR_LENGTH];
 #endif
 
+
+static char zeroSizedReadBuffer[8];
+
 class SFileImageBuffer
 {
 public:
@@ -211,12 +214,19 @@ SFileImage::SFileImage (const SFileImage& in)
   }
 }
 
+
 static int sharedCount=0;
 SFileImageBuffer::SFileImageBuffer(long _fd, long size, bool _writeflag)
 {
   fd = _fd;
   bufferSize = size;
   count = 1;
+  if (size == 0 && !_writeflag) {
+    writeflag = _writeflag;
+    buffer = zeroSizedReadBuffer;
+    sharedCount++;
+    return;
+  }
 #ifdef USE_WINAPI
   writeflag = _writeflag;
   handle = CreateFileMapping ((void*)fd, 0, 
@@ -282,12 +292,18 @@ SFileImageBuffer::~SFileImageBuffer()
     if (fd >= 0)
     {
 #ifdef USE_WINAPI
-      UnmapViewOfFile (buffer);
+      if (buffer != zeroSizedReadBuffer)
+      {
+        UnmapViewOfFile (buffer);
+      }
       CloseHandle (handle);
       CloseHandle ((void*) fd);
 #else
 # ifdef HAVE_MMAP
-      munmap ((char*)buffer, bufferSize);
+      if (buffer != zeroSizedReadBuffer)
+      {
+        munmap ((char*)buffer, bufferSize);
+      }
       ::close (fd);
 # else
 # endif
@@ -295,7 +311,9 @@ SFileImageBuffer::~SFileImageBuffer()
     }
     else
     {
-      delete buffer;
+      if (buffer != zeroSizedReadBuffer) {
+        delete buffer;
+      }
     }
   }
   sharedCount--;
@@ -491,7 +509,7 @@ SFile::SFile (const SString& f) : SIO(SEventSource::FILE)
 }
 
 /**
- * Create a file from a file. The file is reaaly just the name
+ * Create a file from a file. The file is really just the name
  * @param f is the original file
  */
 SFile::SFile (const SFile& f) : SIO(SEventSource::FILE)
@@ -614,6 +632,55 @@ SFile::getOutputStream()
   }
   return out;
 }
+/*
+ * Go to max level and try to find the file in dir.
+ * return true if found.
+*/
+bool
+SFile::recursiveFind (const SString& fileName, SString dirName, int level)
+{
+    if (fileName.size() == 0) return false;
+    if (level < 0) return false;
+    SDir current = SDir(dirName); 
+    if (!current.exists()) return false;
+    if (!current.readable()) return false;
+    SString currentName = current.getName();
+    SString fullPath = currentName;
+
+    fullPath.append("/");
+    fullPath.append(fileName);
+    fullPath.replaceAll("//", "/");
+
+    name = currentName;
+    name.append("/");
+    name.append(fileName);
+    name.replaceAll("//", "/");
+
+    if (size() >= 0) 
+    {
+      //fprintf (stderr, "Found %*.*s level=%d\n", SSARGS (name), level);     
+        return true;
+    }
+    //fprintf (stderr, "Tried %*.*s level=%d\n", SSARGS (name), level);     
+    name = "";
+    if (level == 0) return false;
+
+    SStringVector list = current.list (SDir::SE_DIR);
+
+    for (unsigned int i=0; i<list.size(); i++) 
+    {
+        SString now = currentName;
+        SString newDir = list[i];
+        if (newDir == "." || newDir == "..") continue;
+        now.append ("/");
+        now.append (newDir);
+        if (recursiveFind (fileName, now, level-1))
+        {
+            return true;
+        }
+    }
+    return false;
+}
 
 /**
  * Find the file by the path..
@@ -621,26 +688,22 @@ SFile::getOutputStream()
  * @param pt is the list of paths it could be found.
  * @aram lookcwd is true if we also need to look into current working dir
  */
-SFile::SFile (const SString& filen, const SStringVector& pt, bool lookcwd): SIO(SEventSource::FILE) 
+SFile::SFile (const SString& filen, const SStringVector& dataPath): SIO(SEventSource::FILE) 
 {
-  name = "";
-  /* Serach for file */
-  for (unsigned int i=0; i<pt.size() && size() < 0; i++)
+  for (unsigned int i=0; i<dataPath.size(); i++)
   {
-    SString newName = pt[i];
-    newName.append ("/");
-    newName.append (filen);
-    newName.replaceAll("//", "/");
-    if (!lookcwd && newName == filen)
+    SString element = dataPath[i];
+    if (element.size() > 3 && element.find("/**") > 0) {
+        element.truncate(element.size()-3);
+        if (recursiveFind (filen, element, SB_YUDIT_SUBDIR_LEVEL))
+        {
+            break;
+        }
+    }
+    if (recursiveFind (filen, element, 0))
     {
-      name = "";
-      continue;
+        break;
     }
-    name = newName;
-  }
-  if (size() < 0 && lookcwd)
-  {
-    name =  filen;
   }
 }
 
@@ -657,7 +720,8 @@ SFile::getFileImage(bool writeflag)
 {
   if (map.size()>=0) return map;
   long s = size();
-  if (s <=0) return map;
+// SGC bug if ws s <= 0
+  if (s <0) return map;
 
   SString fn=name;
 #ifdef USE_WINAPI
@@ -845,8 +909,8 @@ SSocket::SSocket (const SSocket& s) : SIO(SEventSource::SOCKET)
 }
 
 /**
- * Assing a socket.
- * do not assign sterams
+ * Assign a socket.
+ * do not assign streams
  */
 SSocket&
 SSocket::operator = (const SSocket& s)
@@ -1239,14 +1303,14 @@ SDir::list (const SStringVector &patterns, SEntry e)
       if (!FindNextFileA (dir, &entryA))
       {
         if (GetLastError()==ERROR_NO_MORE_FILES) break;
-        fprintf (stderr, "some errrors...%d\n", (int) GetLastError());
+        fprintf (stderr, "some errors...%d\n", (int) GetLastError());
         break;
       }
     } else {
       if (!FindNextFileW (dir, &entry))
       {
         if (GetLastError()==ERROR_NO_MORE_FILES) break;
-        fprintf (stderr, "some errrors...%d\n", (int) GetLastError());
+        fprintf (stderr, "some errors...%d\n", (int) GetLastError());
         break;
       }
    }
@@ -1415,7 +1479,7 @@ SDir::exists() const
 
 /**
  * Try to create the directory. 
- * @return true on successs
+ * @return true on success
  */
 bool
 SDir::create () const
@@ -1623,7 +1687,7 @@ SDir::cd (const SString& _newDir)
 
 /**
  * Wait for all waithandles 
- * return false if at leas on of them failed.
+ * return false if at least on of them failed.
  */
 int 
 SPipe::wait ()
@@ -1655,7 +1719,7 @@ SPipe::wait ()
 #endif
 }
 /**
- * Open a pipe and return the file desctiptor to it.
+ * Open a pipe and return the file descriptor to it.
  * Chances are that 
  * @param command the shell command line.
  * @param output is true if we write to the command.
@@ -1818,6 +1882,7 @@ SPipe::openPipe(bool output)
        {
          fprintf (stderr, "fdup failed\n");
        }
+//fprintf (stderr, "child %d %d\n", pp[0], pp[1]);
        close (pp[0]);
      }
      else
@@ -1831,6 +1896,7 @@ SPipe::openPipe(bool output)
 //fprintf (stderr, "return child %d\n", pp[1]);
        close (pp[1]);
      }
+     // it will goo via mytool -pipecmd
      execl ("/bin/sh", "sh", "-c", clString, (char*) 0);
      delete clString;
      exit (127);
