@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -22,6 +22,7 @@
 #include "stoolkit/SBinHashtable.h"
 #include "stoolkit/SIO.h"
 #include "stoolkit/SUtil.h"
+#include "stoolkit/STypes.h" 
 #include <stdio.h>
 
 static SS_UCS4 getMaxBytes (unsigned int inp);
@@ -32,7 +33,7 @@ static SS_UCS4 getMaxBytes (unsigned int inp);
  * Keyboard trasliterations to unicode strings. 
  *
  * This is a tamed version of SBMap.
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 2000-05-12
  */
 
@@ -50,7 +51,7 @@ public:
 
 /**
  * We use the delegate to hide the ugly SBMap and
- * maintain one reference in a chache
+ * maintain one reference in a cache
  */
 SUniMapDelegate::SUniMapDelegate(const SString& n, SBMap* _map)
 {
@@ -69,8 +70,10 @@ typedef SBinHashtable<SUniMapDelegate*> SUniMapHashtable;
  */
 static SUniMapHashtable mapCache;
 
-static SStringVector searchPath("/,maps,../maps,/etc/maps,../mytool/uni,../mytool/kmap,../mytool/mys,mytool/uni,mytool/kmap,mytool/mys,uni,kmap,mys");
-
+static SStringVector searchPath;
+/*
+("/,maps,../maps,/etc/maps,../mytool/uni,../mytool/kmap,../mytool/mys,mytool/uni,mytool/kmap,mytool/mys,uni,kmap,mys");
+*/
 static void _uniAppend (SString* s, SS_UCS4 u);
 
 void
@@ -84,55 +87,11 @@ SUniMap::setPath (const SStringVector& l)
  * property. Always add YUDIT_DATA/data
  */
 void
-SUniMap::guessPath (const SStringVector& files, const SString& property)
+SUniMap::guessPath ()
 {
-  
-  SStringVector outDataPath;
-  for (unsigned int i=0; i<files.size(); i++)
-  {
-    SProperties p;
-    loadProperties (files[i], &p);
-    if (p.get (property))
-    {
-      SStringVector v(p[property], ",:;");
-      for (unsigned int j=0; j<v.size(); j++)
-      {
-         outDataPath.append (v[j]);
-      }
-    }
-  }
-  SString c1 = getHome();
-  c1.append ("/.yudit/data");
-  SString c2 = getPrefix();
-  c2.append ("/data");
-  if (outDataPath.size()!=0)
-  { 
-    outDataPath.append (c1); 
-    outDataPath.append (c2); 
-    setPath (outDataPath);
-  }
-  else
-  {
-    outDataPath.append (c1); 
-    outDataPath.append (c2); 
-    outDataPath.append (getPath());
-    setPath (outDataPath);
-  }
-}
-/**
- * get the path using the "yudit.default.path"
- */
-void
-SUniMap::guessPath()
-{
-  SString c1 = getHome();
-  c1.append ("/.yudit/yudit.properties");
-  SString c2 = getPrefix();
-  c2.append ("/config/yudit.properties");
-  SStringVector v;
-  v.append (c1);
-  v.append (c2);
-  guessPath (v, "yudit.datapath");
+  SStringVector unipath = getUserPath("yudit.datapath", "data");
+//  fprintf (stderr, "uni is %*.*s\n", SSARGS(unipath.join(",")));
+  setPath (unipath);
 }
 
 const SStringVector&
@@ -141,6 +100,7 @@ SUniMap::getPath ()
   return searchPath;
 }
 
+
 /**
  * Vector available maps
  */
@@ -795,13 +755,13 @@ _uniAppend (SString* s, SS_UCS4 u)
   else if (u < 0x10000)
   {
     char u16[32];
-    sprintf (u16, "\\u%04x", (unsigned int) u);
+    snprintf (u16, 32, "\\u%04x", (unsigned int) u);
     s->append (u16);
   }
   else 
   {
     char u32[32];
-    sprintf (u32, "\\U%04x", (unsigned int) u);
+    snprintf (u32, 32, "\\U%04x", (unsigned int) u);
     s->append (u32);
   }
 }
