@@ -1,7 +1,7 @@
 /**
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
+ *  GNU Copyright (C) 1997-2023  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -175,6 +175,23 @@ static SRendClass::RType  l_tamil_reorder_guide[] =
   SRendClass::None
 };
 
+/* FIXME */
+static SRendClass::RType  l_sinhala_reorder_guide[] =
+{
+  SRendClass::Cpre,
+  SRendClass::Cbase,
+  SRendClass::VO,  
+  SRendClass::Mabove,
+  SRendClass::Mpost,
+  SRendClass::Cbelow,
+  SRendClass::Cpost,
+  SRendClass::LMpost, // For Kannada 0CD5 and 0CD6 is not in indic.txt
+  SRendClass::Creph,
+  SRendClass::CMreph,
+  SRendClass::VMpost,
+  SRendClass::None
+};
+
 /*!
  * \brief Find out if the script is supported by this 
  * shaping engine.
@@ -216,7 +233,7 @@ SScriptProcessor::isSupported (SS_UCS4 c) const
   /* Malayalam */
   if (c >= 0x0d00 && c<= 0x0d7f) return true;
   /* Sinhala - not supported, need to get info */
-  if (c >= 0x0d80 && c<= 0x0dff) return false; //<<<<
+  if (c >= 0x0d80 && c<= 0x0dff) return true; //<<<<
   /* Thai */
   if (c >= 0x0e00 && c<= 0x0e7f) return true;
   /* Lao */
@@ -279,7 +296,7 @@ SScriptProcessor::put (const SS_UCS4* in, unsigned int len, bool is_begin)
 
   unsigned int reallen = 0; // Real length.
   unsigned int min = 0;     // minimum range (inclusive)
-  unsigned int max = 0;     // maxinum range (non-inclusive)
+  unsigned int max = 0;     // maximum range (non-inclusive)
   m_script_has_reph = false;
   m_ra = 0;
   const SRendClass::RType* order = 0; 
@@ -389,7 +406,7 @@ SScriptProcessor::put (const SS_UCS4* in, unsigned int len, bool is_begin)
     m_script = SC_SINHALA;
     m_otfScript = "sinh";
     min = 0x0d80; max = 0x0e00;
-    order = l_devanagari_gujarati_reorder_guide;
+    order = l_sinhala_reorder_guide;
     m_script_has_reph = true;
   }
   // Thai
@@ -848,7 +865,7 @@ SScriptProcessor::gsub (const char* feature)
       }
     }
   }
-  delete lig;
+  delete[] lig;
 }
 
 /*!
@@ -1419,7 +1436,8 @@ SScriptProcessor::findBaseConsonant()
   /*
    * Modify first type if necassary (Creph)
    */
-  if (typeFirst == SRendClass::Cra || SRendClass::Creph)
+  // SGC IOS
+  if (typeFirst == SRendClass::Cra || typeFirst == SRendClass::Creph)
   {
     if (has_vo && m_script_has_reph)
     {
@@ -1795,7 +1813,7 @@ SScriptProcessor::reorderIndic ()
       m_out_type.replace (c_last_pos, SRendClass::Clast);
   }
 
-  // Add additional properries for rendering.
+  // Add additional properties for rendering.
 
   unsigned int ucount =0;
   for (i=0; i< m_in.size(); i++)
@@ -1883,7 +1901,7 @@ SScriptProcessor::apply ()
     gsub_guide = SStringVector("nukt,akhn,rphf,blwf,half,pstf,vatu,pres,abvs,blws,psts,haln");
     gpos_guide = SStringVector("abvm,blwm,dist");
     break;
-  // Go with the generic recipie
+  // Go with the generic recipe
   case SC_ORIYA:
   case SC_KANNADA:
   case SC_MALAYALAM:
